#ifndef _CLAPIC_
#define _CLAPIC_

struct Ufo;
#pragma save,call(seg_name => clareng)

/*-----------------14/05/96 11:35-------------------
  Picture type header file, taken from cla.h
  To be shared by compiler and library
--------------------------------------------------*/

#define MaxPicLen  100

typedef struct
{
  unsigned char   c;      // display character at this place
  unsigned char   phold;  // character place holder type
} PicPlaces;

#define PLACECONST      0   // constant char PicPlaces.phold
#define PLACECOMMA      1   // comma/grouping PicPlaces.phold
#define PLACEDECIMAL    2   // decimal point PicPlaces.phold
#define PLACEFILL       3   // fill character PicPlaces.phold
#define PLACEBLANK      4   // blank fill PicPlaces.phold
#define PLACESIGN       5   // sign PicPlaces.phold
#define PLACECURRENCY   6   // currency PicPlaces.phold
#define PLACEMONTH      7   // month (numeric) PicPlaces.phold
#define PLACEDAY        8   // day PicPlaces.phold
#define PLACEYEAR       9   // year PicPlaces.phold
#define PLACEMONTHA     10  // month (alpha) PicPlaces.phold
#define PLACEHOUR       11  // hour PicPlaces.phold
#define PLACEMINUTE     12  // minute PicPlaces.phold
#define PLACESECOND     13  // second PicPlaces.phold
#define PLACEALPHA      14  // alpha PicPlaces.phold
#define PLACEUPPER      15  // upper case PicPlaces.phold
#define PLACELOWER      16  // lower case PicPlaces.phold
#define PLACEANY        17  // any character PicPlaces.phold
#define PLACESTOP       18  // stop format PicPlaces.phold
#define PLACESEP        19  // seperator PicPlaces.phold
#define PLACEAMPM       20
#define PLACESIGNE      21
#define PLACEVDECIMAL   22
#define PLACEKSTOP      23

typedef struct
{
  unsigned    token:3;    // picture token type
  unsigned    dtype:3;    // data type for conversion
  unsigned    blnk:1;     // blank switch (run)
  unsigned    width:8;    // width of picture (num. places
#if __32BIT__
  unsigned    prod_ilka:16; // Work around compiler bug
#endif
} PicHead;

enum tok_type { PTOKN, PTOKE, PTOKD, PTOKT, PTOKP, PTOKS, PTOKK };

#define PICSTRING  0   // convert to string - PicDesc.dtype
#define PICLONG    1   // convert to long - PicDesc.dtype
#define PICREAL    2   // convert to real - PicDesc.dtype
#define PICDECIMAL 3   // convert to decimal - PicDesc.dtype

typedef struct {
  int         iplc;       // integer place count
  int         dplc;       // decimal place count
  int         decpos;     // decimal point position
  int         decchar;    // type of decimal point
  int         moc;        // month place count
  int         yrc;        // year place count
  int         timec;      // time place count
  int         comc;       // number of commas     ***** New

  char        exp;        // exponent indicator
  char        expos;      // exponent position
  char        blank;      // blank fill indicator
  int         inspos;
  int         ovrpos;
  int         lastpos;
  int         slots;
} PicCounts;

class PicDesc
{
  public:
    PicDesc() { Clear(); };
    PicDesc(const char *s) { Init(s); };
    void Clear();
    void Init(const char *s);
    int  Valid() { return h.width != 0; };
    // From gets corrupted and is expected to have at least 5 bytes spare
    void FormatPict(char *dest, unsigned l, int dec,int sign,char *from);
    char *DeformatPict(const char *src, unsigned l);
    void Count(PicCounts&);
    void StoreLong(char *s, unsigned l, long v);
    void StoreReal(char *s, unsigned l, double v);
    void StoreDec(char *s, unsigned l); // Taken from tods
    void PushDeformatted(const char *s);
    Ufo  *GiveUfo(void *m,char copy);
    PicHead     h;
    char        epoch_shift;// When is 'beginning' of time
    char        date_val;
    PicPlaces   place[MaxPicLen]; // array of width # of PicPlaces
    const char  *picture;
};

#define INTL_MAX_LONG_DATE_WIDTH  32
#define INTL_MAX_SHORT_DATE_WIDTH 16
#define INTL_MAX_LONG_TIME_WIDTH  16
#define INTL_MAX_SHORT_TIME_WIDTH 12

extern "Modula2" {

unsigned Cla$storepict(const char *ptok,PicDesc *pdesc);
PicHead Cla$checkpict(const char *ptok);
unsigned Cla$getpicturewidth(const char * s);
int      Cla$pictureisvalid(const char *s);

}

#pragma restore

#endif

