/*-----------------01/11/95 10:43-------------------
Clarion View descriptor definition
Note that this class does not deal with the fcb list
--------------------------------------------------*/
#define MAX_NEST 20

// Procedures called in wrong sequence
#define VERR_BADSEQUENCE 1
//More components supplied that required
#define VERR_COMPONENTS  2
//Unseting more joins than setting
#define VERR_UNSET       3
//Bad join number
#define VERR_SET         4
//Nesting joins too deep
#define VERR_MAXNEST     5
//Number out of range
#define VERR_BIGNUM      6
//Empty join incountered
#define VERR_NOFIELD     7

typedef unsigned short field_no;
typedef char           cond_no;
typedef char           join_no;

#define NOT_A_FIELD     ((field_no)0xFFFF)
#define NOT_A_MEMO      NOT_A_FIELD
#define JOIN_CONDITION  ((cond_no)0xFF)
#define ALL_FIELDS      NOT_A_FIELD
#define ViewFlag_Inner  1

class view_descriptor
{
  public:
    void setup(char *block);
    // Used for writing descriptor
    void file();                    // Start writing view descriptor
    void join(cond_no components);
    void join_condition(char *condition); // Both do an implicit set_join
    void join_component(field_no);
    void project_field(field_no);
    void project_memo(field_no);
    void project_all();
    void set_flag(char);
    // Common routines
    void unset_join();              // Used to 'come out' a join
    // Used for reading vcb
    void set_join(join_no join);    // Used to 'move into' a join. 0 for outer file
    char get_flags();               // Flags for given join, after components but before fields
    field_no fields();              // Fields in current join - finger points to fields  (ALL_FIELDS if all fields included)
    field_no field(field_no);       // Returns field number NOT_A_FIELD for memo
    field_no memo(field_no);        // NOT_A_MEMO for normal field
    join_no joins();                // Number of nested joins
    char *condition();              // Null for normal component join
    cond_no components();           // JOIN_CONDITION means join condition
    field_no component(cond_no);
    void print();
    void print_join();
  protected:
    virtual void extend_block(unsigned); // Needed for writing block only
    virtual void error(unsigned);   // Return error code
    virtual void out_char(char ){};  // Needed for print only
  private:
    enum wait_type { st_waiting, st_components, st_flags, st_fields, st_joins } waiting;
    void skip_joinrec();
    void skip_tojoins();
    void skip_tofields();
    void skip_toflags();
    void skip_fields(unsigned);
    void null_join();
    void make_state( wait_type );
    char *finger,*inblock;
    cond_no expected_components;
    char outtab,dotab;
    void OutText(char *);
    void OutNumber(unsigned);
    void OutLine();
    void OutSetTab(char);
    union
    {
      join_no *join_count;
      field_no *field_count;
      cond_no *component_count;
    } nest[MAX_NEST];
    char curnest;
};

#ifndef COMPILER
#include <clwclib.h>
class ViewDesc : public view_descriptor
{
  public:
    ViewDesc( char *block = NULL );
    ~ViewDesc();
    char *YieldBlock();
    void Desc2Stack();
  protected:
    virtual void error(unsigned n);
    virtual void out_char(char c);
    virtual void extend_block(unsigned n);
  private:
    char *hold_block;
    char newed;
    unsigned size;
};

#endif

