/*------------------------------------------------------------------
 *
 * Hash Table functions for 32-bit WSL.
 *
 *------------------------------------------------------------------*/
#ifndef _WSLHASH_INCL
#define _WSLHASH_INCL

#ifdef __cplusplus
class HashTable;

class HashElement
{
    friend HashTable;
  private:
    void    *key;             /* points to the key for this element */
    unsigned link;            /* Number of times it's been added    */
    unsigned hash;            /* Hash value for this element        */

  public:
    HashElement();
    virtual ~HashElement();

    void        *HKey() const       { return key; }
    HashElement *HLink();
    unsigned     HLinkCount() const { return link; }
};

class HashTable
{
  private:
    HashElement * *table;
    int tablesize;
    int tablecount;
    int keysize;
    unsigned char   nocase;

  private:
    BOOL     keyeq(const void *key1, const void *key2, int ksize) const;
    unsigned hash(const void *key, int ksize) const;

    unsigned find(unsigned hashval, const void *key) const;
    void     expand();

  protected:
    virtual HashElement *NewHashElement() const = 0;

  public:
    HashTable(int keysize);
    HashTable::~HashTable();

    HashElement *HAdd(const void *key);
    unsigned     HCount() const { return tablecount; }
    void         HDelete(HashElement *a);
    HashElement *HFind(const void *key) const;
    HashElement *HNext(const HashElement *) const;
    void         Kill(void);
    void         setcase(BOOL val)       { nocase = val; }
};

template <class T, unsigned int K> class HashTableOf : public HashTable
{
    friend T;
  protected:
    virtual HashElement *NewHashElement() const { return new T; }
  public:
    HashTableOf<T,K>() : HashTable(K) {};
    T *HAdd(const void *key) { return (T *) HashTable::HAdd(key); }
    T *HFind(const void *key) const { return (T *) HashTable::HFind(key); }
    T *HNext(const HashElement *i) const { return (T *) HashTable::HNext(i); }
};

class Atom : public HashElement
{
  public:
    Atom * HLink() { return (Atom *) HashElement::HLink(); };
    unsigned    GetAtomName (char *name, unsigned namelen) const;
    const char *GetAtomNamePtr() const;
};

typedef Atom *_ATOM;

class AtomTable : public HashTableOf<Atom, 0U>
{
  public:
    _ATOM AddAtom(const char *name) { return (_ATOM) HAdd(name); };
};


#endif

#endif  // _WSLHASH_INCL